% ==== Demo =======
% ========= CITATION ============
% Yicong Zhou and Yantao Wei, 
% "Learning Hierarchical Spectral-Spatial Features for Hyperspectral Image Classification" IEEE Transactions on Cybernetics, 2015. 

% Yantao Wei [yantaowei@mail.ccnu.edu.cn]
% Please email me if you find bugs, or have suggestions or questions!
clear all; close all; clc; 
%% 
% load data
load('./Indian_pines/Indian_pines_corrected'); 
load('./Indian_pines/Indian_pines_gt'); 
% =======================================
Mi=min(min(min(indian_pines_corrected)));
Ma=max(max(max(indian_pines_corrected)));
for i=1:size(indian_pines_corrected,1)
    for j=1:size(indian_pines_corrected,2)
         indian_pines_corrected(i,j,:)=(indian_pines_corrected(i,j,:)-Mi)/(Ma-Mi);
    end
end

[C D]=find(indian_pines_gt>0);

TestingAccuracy=zeros(1,10);
for Iter=1:10
NClass=16;
TrnLabels=[];
TestLabels=[];
Tr_idx_C=[];
Te_idx_C=[];
Te_idx_R=[];
Tr_idx_R=[];
for i=1:NClass   
    [R C]=find(indian_pines_gt==i);
    Num=ceil(numel(C)*0.01);
    idx_rand=randperm(numel(C));
    Tr_idx_C=[Tr_idx_C C(idx_rand(1:Num))'];
    Tr_idx_R=[Tr_idx_R R(idx_rand(1:Num))'];
    Te_idx_R=[Te_idx_R R(idx_rand(Num+1:end))'];
    Te_idx_C=[Te_idx_C C(idx_rand(Num+1:end))'];
    TrnLabels=[TrnLabels ones(1,Num)*i];
    TestLabels=[TestLabels ones(1,numel(C)-Num)*i]; 
end
% ===========================================================

%% parameters
DLNet.NumStages = 6;
DLNet.PatchSize = [1 21,1,5,1,5];
DLNet.NumFilters = [15 5,15,5,15,5];
fprintf('\n ====== Parameters ======= \n')
DLNet

%% Training

fprintf('\n ======Training ======= \n')
[f,V] = DLNet_train(indian_pines_corrected,TrnLabels,Tr_idx_R,Tr_idx_C,DLNet); % 
ftrain=zeros(size(f,3),numel(Tr_idx_R));
for i=1:numel(Tr_idx_R)
    ftrain(:,i)=reshape(f(Tr_idx_R(i),Tr_idx_C(i),:),size(f,3),1);
end
clear TrnData_ImgCell; 


%% Testing 
fprintf('\n ====== Testing ======= \n')
ft = DLNet_Test(indian_pines_corrected,V,DLNet);
ftest=zeros(size(ft,3),numel(Te_idx_R));
for i=1:numel(Te_idx_R)
    ftest(:,i)=reshape(ft(Te_idx_R(i),Te_idx_C(i),:),size(ft,3),1);
end

pa1=1000;
pa2=0.01;

tic
[TrainingTime, TestingTime, TrainingAccuracy, TestingAccuracy(Iter),label_index_expected(Iter,:)] = elm_kernel(ftrain, TrnLabels,ftest,TestLabels,1, pa1, 'RBF_kernel',pa2);
toc

[kappa(Iter), acc(:,Iter), acc_O(Iter), acc_A(Iter)] = evaluate_results(label_index_expected(Iter,:), TestLabels);
end



    